"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const adm_zip_1 = tslib_1.__importDefault(require("adm-zip"));
const map_1 = tslib_1.__importDefault(require("lodash/map"));
const omit_1 = tslib_1.__importDefault(require("lodash/omit"));
const find_1 = tslib_1.__importDefault(require("lodash/find"));
const form_data_1 = tslib_1.__importDefault(require("form-data"));
const filter_1 = tslib_1.__importDefault(require("lodash/filter"));
const includes_1 = tslib_1.__importDefault(require("lodash/includes"));
const isEmpty_1 = tslib_1.__importDefault(require("lodash/isEmpty"));
const path_1 = require("path");
const cli_utilities_1 = require("@contentstack/cli-utilities");
const fs_1 = require("fs");
const util_1 = require("../util");
const base_class_1 = tslib_1.__importDefault(require("./base-class"));
const fs_2 = require("../util/fs");
const graphql_1 = require("../graphql");
const launch_1 = require("../types/launch");
const config_1 = tslib_1.__importDefault(require("../config"));
class FileUpload extends base_class_1.default {
    /**
     * @method run
     *
     * @return {*}  {Promise<void>}
     * @memberof FileUpload
     */
    async run() {
        if (this.config.isExistingProject) {
            const environment = await this.getEnvironment();
            await this.handleExistingProject(environment.uid);
        }
        else {
            await this.handleNewProject();
        }
        this.prepareLaunchConfig();
        await this.showLogs();
        if (this.config.currentDeploymentStatus === launch_1.DeploymentStatus.FAILED) {
            this.exit(1);
        }
        this.showDeploymentUrl();
        this.showSuggestion();
    }
    async handleExistingProject(environment) {
        await this.initApolloClient();
        let redeployLatest = this.config['redeploy-latest'];
        let redeployLastUpload = this.config['redeploy-last-upload'];
        if (!redeployLatest && !redeployLastUpload) {
            await this.confirmRedeployment();
            const latestRedeploymentConfirmed = await this.confirmLatestRedeployment();
            redeployLatest = latestRedeploymentConfirmed;
            redeployLastUpload = !latestRedeploymentConfirmed;
        }
        if (redeployLastUpload && redeployLatest) {
            this.log('redeploy-last-upload and redeploy-latest flags are not supported together.', 'error');
            this.exit(1);
        }
        let uploadUid;
        if (redeployLatest) {
            const signedUploadUrlData = await this.createSignedUploadUrl();
            uploadUid = signedUploadUrlData.uploadUid;
            const { zipName, zipPath } = await this.archive();
            await this.uploadFile(zipName, zipPath, signedUploadUrlData);
        }
        await this.createNewDeployment(true, environment, uploadUid);
    }
    async confirmRedeployment() {
        const redeploy = await cli_utilities_1.cliux.inquire({
            type: 'confirm',
            name: 'deployLatestCommit',
            message: 'Do you want to redeploy this existing Launch project?',
        });
        if (!redeploy) {
            this.log('Project redeployment aborted.', 'info');
            this.exit(1);
        }
    }
    async confirmLatestRedeployment() {
        const choices = [
            ...(0, map_1.default)(config_1.default.supportedFileUploadMethods, (fileUploadMethod) => ({
                value: fileUploadMethod,
                name: `Redeploy with ${fileUploadMethod}`,
            }))
        ];
        const selectedFileUploadMethod = await cli_utilities_1.cliux.inquire({
            choices: choices,
            type: 'search-list',
            name: 'fileUploadMethod',
            message: 'Choose a redeploy method to proceed',
        });
        if (selectedFileUploadMethod === launch_1.FileUploadMethod.LastFileUpload) {
            return false;
        }
        return true;
    }
    async handleNewProject() {
        const uploadUid = await this.prepareAndUploadNewProjectFile();
        await this.createNewProject(uploadUid);
    }
    /**
     * @method createNewProject - Create new launch project
     *
     * @return {*}  {Promise<void>}
     * @memberof FileUpload
     */
    async createNewProject(uploadUid) {
        const { framework, projectName, buildCommand, outputDirectory, environmentName, serverCommand } = this.config;
        await this.apolloClient
            .mutate({
            mutation: graphql_1.importProjectMutation,
            variables: {
                project: {
                    projectType: 'FILEUPLOAD',
                    name: projectName,
                    fileUpload: { uploadUid },
                    environment: {
                        frameworkPreset: framework,
                        outputDirectory: outputDirectory,
                        name: environmentName || 'Default',
                        environmentVariables: (0, map_1.default)(this.envVariables, ({ key, value }) => ({ key, value })),
                        buildCommand: buildCommand === undefined || buildCommand === null ? 'npm run build' : buildCommand,
                        serverCommand: serverCommand === undefined || serverCommand === null ? 'npm run start' : serverCommand,
                    },
                },
                skipGitData: true,
            },
        })
            .then(({ data: { project } }) => {
            this.log('New project created successfully', 'info');
            const [firstEnvironment] = project.environments;
            this.config.currentConfig = project;
            this.config.currentConfig.deployments = (0, map_1.default)(firstEnvironment.deployments.edges, 'node');
            this.config.currentConfig.environments[0] = (0, omit_1.default)(this.config.currentConfig.environments[0], ['deployments']);
        })
            .catch(async (error) => {
            const canRetry = await this.handleNewProjectCreationError(error);
            if (canRetry) {
                return this.createNewProject(uploadUid);
            }
        });
    }
    /**
     * @method prepareForNewProjectCreation - prepare necessary data for new project creation
     *
     * @return {*}  {Promise<void>}
     * @memberof FileUpload
     */
    async prepareAndUploadNewProjectFile() {
        var _a, _b;
        const { name, framework, environment, 'build-command': buildCommand, 'out-dir': outputDirectory, 'variable-type': variableType, 'env-variables': envVariables, 'server-command': serverCommand, alias, } = this.config.flags;
        const { token, apiKey } = (_a = cli_utilities_1.configHandler.get(`tokens.${alias}`)) !== null && _a !== void 0 ? _a : {};
        this.config.selectedStack = apiKey;
        this.config.deliveryToken = token;
        // this.fileValidation();
        await this.selectOrg();
        const signedUploadUrlData = await this.createSignedUploadUrl();
        const { zipName, zipPath, projectName } = await this.archive();
        await this.uploadFile(zipName, zipPath, signedUploadUrlData);
        this.config.projectName =
            name ||
                (await cli_utilities_1.cliux.inquire({
                    type: 'input',
                    name: 'projectName',
                    message: 'Project Name',
                    default: projectName,
                    validate: this.inquireRequireValidation,
                }));
        this.config.environmentName =
            environment ||
                (await cli_utilities_1.cliux.inquire({
                    type: 'input',
                    default: 'Default',
                    name: 'environmentName',
                    message: 'Environment Name',
                    validate: this.inquireRequireValidation,
                }));
        if (framework) {
            this.config.framework = ((0, find_1.default)(this.config.listOfFrameWorks, {
                name: framework,
            }).value || '');
            (0, util_1.print)([
                { message: '?', color: 'green' },
                { message: 'Framework Preset', bold: true },
                { message: this.config.framework, color: 'cyan' },
            ]);
        }
        else {
            await this.detectFramework();
        }
        this.config.buildCommand =
            buildCommand ||
                (await cli_utilities_1.cliux.inquire({
                    type: 'input',
                    name: 'buildCommand',
                    message: 'Build Command',
                    default: this.config.framework === 'OTHER' ? null : 'npm run build',
                }));
        this.config.outputDirectory =
            outputDirectory ||
                (await cli_utilities_1.cliux.inquire({
                    type: 'input',
                    name: 'outputDirectory',
                    message: 'Output Directory',
                    default: this.config.outputDirectories[((_b = this.config) === null || _b === void 0 ? void 0 : _b.framework) || 'OTHER'],
                }));
        if (this.config.framework && this.config.supportedFrameworksForServerCommands.includes(this.config.framework)) {
            this.config.serverCommand =
                serverCommand ||
                    (await cli_utilities_1.cliux.inquire({
                        type: 'input',
                        name: 'serverCommand',
                        message: 'Server Command',
                    }));
        }
        this.config.variableType = variableType;
        this.config.envVariables = envVariables;
        await this.handleEnvImportFlow();
        return signedUploadUrlData.uploadUid;
    }
    /**
     * @method fileValidation - validate the working directory
     *
     * @memberof FileUpload
     */
    fileValidation() {
        const basePath = this.config.projectBasePath;
        const packageJsonPath = (0, path_1.resolve)(basePath, 'package.json');
        if (!(0, fs_1.existsSync)(packageJsonPath)) {
            this.log('Package.json file not found.', 'info');
            this.exit(1);
        }
    }
    /**
     * @method archive - Archive the files and directory to be uploaded for launch project
     *
     * @return {*}
     * @memberof FileUpload
     */
    async archive() {
        cli_utilities_1.ux.action.start('Preparing zip file');
        const projectName = (0, path_1.basename)(this.config.projectBasePath);
        const zipName = `${Date.now()}_${projectName}.zip`;
        const zipPath = (0, path_1.resolve)(this.config.projectBasePath, zipName);
        const zip = new adm_zip_1.default();
        const zipEntries = (0, filter_1.default)(await (0, fs_2.getFileList)(this.config.projectBasePath, true, true), (entry) => !(0, includes_1.default)(this.config.fileUploadConfig.exclude, entry) && !(0, includes_1.default)(entry, '.zip'));
        for (const entry of zipEntries) {
            const entryPath = `${this.config.projectBasePath}/${entry}`;
            const state = (0, fs_1.statSync)(entryPath);
            switch (true) {
                case state.isDirectory(): // NOTE folder
                    zip.addLocalFolder(entryPath, entry);
                    break;
                case state.isFile(): // NOTE check is file
                    zip.addLocalFile(entryPath);
                    break;
            }
        }
        const status = await zip.writeZipPromise(zipPath).catch(() => {
            this.log('Zipping project process failed! Please try again.');
            this.exit(1);
        });
        if (!status) {
            this.log('Zipping project process failed! Please try again.');
            this.exit(1);
        }
        cli_utilities_1.ux.action.stop();
        return { zipName, zipPath, projectName };
    }
    /**
     * @method createSignedUploadUrl - create pre signed url for file upload
     *
     * @return {*}  {Promise<SignedUploadUrlData>}
     * @memberof FileUpload
     */
    async createSignedUploadUrl() {
        try {
            const result = await this.apolloClient.mutate({ mutation: graphql_1.createSignedUploadUrlMutation });
            const signedUploadUrlData = result.data.signedUploadUrl;
            this.config.uploadUid = signedUploadUrlData.uploadUid;
            return signedUploadUrlData;
        }
        catch (error) {
            this.log('Something went wrong. Please try again.', 'warn');
            if (error instanceof Error) {
                this.log(error.message, 'error');
            }
            this.exit(1);
            return {};
        }
    }
    /**
     * @method uploadFile - Upload file in to s3 bucket
     *
     * @param {string} fileName
     * @param {PathLike} filePath
     * @return {*}  {Promise<void>}
     * @memberof FileUpload
     */
    async uploadFile(fileName, filePath, signedUploadUrlData) {
        const { uploadUrl, fields, headers, method } = signedUploadUrlData;
        const formData = new form_data_1.default();
        if (!(0, isEmpty_1.default)(fields)) {
            for (const { formFieldKey, formFieldValue } of fields) {
                formData.append(formFieldKey, formFieldValue);
            }
            formData.append('file', (0, fs_1.createReadStream)(filePath), fileName);
            await this.submitFormData(formData, uploadUrl);
        }
        else if (method === 'PUT') {
            await this.uploadWithHttpClient(filePath, uploadUrl, headers);
        }
    }
    async submitFormData(formData, uploadUrl) {
        cli_utilities_1.ux.action.start('Starting file upload...');
        try {
            await new Promise((resolve, reject) => {
                formData.submit(uploadUrl, (error, res) => {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve();
                    }
                });
            });
            cli_utilities_1.ux.action.stop();
        }
        catch (error) {
            cli_utilities_1.ux.action.stop('File upload failed!');
            this.log('File upload failed. Please try again.', 'error');
            if (error instanceof Error) {
                this.log(error.message, 'error');
            }
            this.exit(1);
        }
    }
    async uploadWithHttpClient(filePath, uploadUrl, headers) {
        cli_utilities_1.ux.action.start('Starting file upload...');
        const httpClient = new cli_utilities_1.HttpClient();
        const file = (0, fs_1.readFileSync)(filePath);
        // Convert headers array to a headers object
        const headerObject = headers === null || headers === void 0 ? void 0 : headers.reduce((acc, { key, value }) => {
            acc[key] = value;
            return acc;
        }, {});
        try {
            httpClient.headers({ 'Content-Type': 'application/zip' });
            if (headerObject !== undefined)
                httpClient.headers(headerObject);
            const response = (await httpClient.put(uploadUrl, file));
            const { status } = response;
            if (status >= 200 && status < 300) {
                cli_utilities_1.ux.action.stop();
            }
            else {
                cli_utilities_1.ux.action.stop('File upload failed!');
                this.log('File upload failed. Please try again.', 'error');
                this.log(`Error: ${status}, ${response === null || response === void 0 ? void 0 : response.statusText}`, 'error');
                this.exit(1);
            }
        }
        catch (error) {
            cli_utilities_1.ux.action.stop('File upload failed!');
            this.log('File upload failed. Please try again.', 'error');
            if (error instanceof Error) {
                this.log(`Error: ${error.message}`, 'error');
            }
            this.exit(1);
        }
    }
}
exports.default = FileUpload;
