"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const map_1 = tslib_1.__importDefault(require("lodash/map"));
const cli_utilities_1 = require("@contentstack/cli-utilities");
const config_1 = tslib_1.__importDefault(require("../../config"));
const base_command_1 = require("../../base-command");
const adapters_1 = require("../../adapters");
class Launch extends base_command_1.BaseCommand {
    async run() {
        if (!this.flags.init) {
            await this.getConfig();
        }
        await this.prepareApiClients();
        this.$event.on('provider-changed', () => {
            this.manageFlowBasedOnProvider();
        });
        // NOTE pre-check: manage flow and set the provider value
        await this.preCheckAndInitConfig();
        await this.manageFlowBasedOnProvider();
    }
    /**
     * @method manageFlowBasedOnProvider - Manage launch flow based on provider (GitHb, FileUpload etc.,)
     *
     * @return {*}  {Promise<void>}
     * @memberof Launch
     */
    async manageFlowBasedOnProvider() {
        const adapterConstructorInputs = {
            log: this.log,
            exit: process.exit,
            $event: this.$event,
            config: this.sharedConfig,
            apolloClient: this.apolloClient,
            managementSdk: this.managementSdk,
            analyticsInfo: this.context.analyticsInfo,
        };
        switch (this.sharedConfig.provider) {
            case 'GitHub':
                await new adapters_1.GitHub(adapterConstructorInputs).run();
                break;
            case 'FileUpload':
                await new adapters_1.FileUpload(adapterConstructorInputs).run();
                break;
            default:
                await this.preCheck.connectToAdapterOnUi();
                break;
        }
    }
    /**
     * @method preCheckAndInitConfig - prepare and initialize the configurations
     *
     * @return {*}  {Promise<void>}
     * @memberof Launch
     */
    async preCheckAndInitConfig() {
        this.preCheck = new adapters_1.PreCheck({
            log: this.log,
            exit: process.exit,
            $event: this.$event,
            config: this.sharedConfig,
            apolloClient: this.apolloClient,
            managementSdk: this.managementSdk,
            analyticsInfo: this.context.analyticsInfo,
        });
        await this.preCheck.run(!this.flags.type);
    }
}
exports.default = Launch;
Launch.description = 'Launch related operations';
Launch.examples = [
    '<%= config.bin %> <%= command.id %>',
    '<%= config.bin %> <%= command.id %> --data-dir <path/of/current/working/dir>',
    '<%= config.bin %> <%= command.id %> --config <path/to/launch/config/file>',
    '<%= config.bin %> <%= command.id %> --type <options: GitHub|FileUpload>',
    '<%= config.bin %> <%= command.id %> --data-dir <path/of/current/working/dir> --type <options: GitHub|FileUpload>',
    '<%= config.bin %> <%= command.id %> --data-dir <path/of/current/working/dir> --redeploy-latest',
    '<%= config.bin %> <%= command.id %> --data-dir <path/of/current/working/dir> --redeploy-last-upload',
    '<%= config.bin %> <%= command.id %> --config <path/to/launch/config/file> --type <options: GitHub|FileUpload>',
    '<%= config.bin %> <%= command.id %> --environment=<value> --redeploy-latest',
    '<%= config.bin %> <%= command.id %> --config <path/to/launch/config/file> --type <options: GitHub|FileUpload> --name=<value> --environment=<value> --branch=<value> --build-command=<value> --framework=<option> --org=<value> --out-dir=<value>',
    '<%= config.bin %> <%= command.id %> --config <path/to/launch/config/file> --type <options: GitHub|FileUpload> --name=<value> --environment=<value> --branch=<value> --build-command=<value> --framework=<option> --org=<value> --out-dir=<value> --server-command=<value>',
    '<%= config.bin %> <%= command.id %> --config <path/to/launch/config/file> --type <options: GitHub|FileUpload> --name=<value> --environment=<value> --branch=<value> --build-command=<value> --framework=<option> --org=<value> --out-dir=<value> --variable-type="Import variables from a stack" --alias=<value>',
    '<%= config.bin %> <%= command.id %> --config <path/to/launch/config/file> --type <options: GitHub|FileUpload> --name=<value> --environment=<value> --branch=<value> --build-command=<value> --framework=<option> --org=<value> --out-dir=<value> --variable-type="Manually add custom variables to the list" --env-variables="APP_ENV:prod, TEST_ENV:testVal"',
];
Launch.flags = {
    type: cli_utilities_1.Flags.string({
        options: [...config_1.default.supportedAdapters, 'FileUpload'],
        description: '[optional] Type of adapters. <options: GitHub|FileUpload>',
    }),
    framework: cli_utilities_1.Flags.string({
        options: (0, map_1.default)(config_1.default.listOfFrameWorks, 'name'),
        description: '[optional] Type of framework. <options: Gatsby|NextJS|Other>',
    }),
    org: cli_utilities_1.Flags.string({
        description: '[optional] Provide the organization UID to create a new project or deployment.',
    }),
    name: cli_utilities_1.Flags.string({
        char: 'n',
        description: '[optional] Name of the project.',
    }),
    environment: cli_utilities_1.Flags.string({
        char: 'e',
        description: '[optional] Environment name for the Launch project.',
    }),
    branch: cli_utilities_1.Flags.string({
        description: '[optional] GitHub branch name.',
    }),
    'build-command': cli_utilities_1.Flags.string({
        description: '[optional] Build Command.',
    }),
    'out-dir': cli_utilities_1.Flags.string({
        description: '[optional] Output Directory.',
    }),
    'server-command': cli_utilities_1.Flags.string({
        description: '[optional] Server Command.',
    }),
    'variable-type': cli_utilities_1.Flags.string({
        options: [...config_1.default.variablePreparationTypeOptions],
        description: '[optional] Provide a variable type. <options: Import variables from a stack|Manually add custom variables to the list|Import variables from the local env file>',
    }),
    'show-variables': cli_utilities_1.Flags.boolean({
        hidden: true,
        default: false,
        description: '[optional, Hidden] Show variable values on the UI.',
    }),
    init: cli_utilities_1.Flags.boolean({
        hidden: true,
        description: '[optional, Hidden] Reinitialize the project if it is an existing Launch project.',
    }),
    alias: cli_utilities_1.Flags.string({
        char: 'a',
        description: '[optional] Alias (name) for the delivery token.',
    }),
    'env-variables': cli_utilities_1.Flags.string({
        description: '[optional] Provide the environment variables in the key:value format, separated by comma. For example: APP_ENV:prod, TEST_ENV:testVal.',
    }),
    'redeploy-latest': cli_utilities_1.Flags.boolean({
        description: '[optional] Redeploy latest commit/code',
        default: false,
    }),
    'redeploy-last-upload': cli_utilities_1.Flags.boolean({
        description: '[optional] Redeploy with last file upload',
        default: false,
    }),
};
