"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudFunctions = void 0;
const tslib_1 = require("tslib");
const dotenv_1 = tslib_1.__importDefault(require("dotenv"));
const express_1 = tslib_1.__importDefault(require("express"));
const path_1 = tslib_1.__importDefault(require("path"));
const cloud_functions_validator_1 = require("./cloud-functions-validator");
const constants_1 = require("./constants");
const cloud_function_errors_1 = require("./errors/cloud-function.errors");
const os_helper_1 = require("./os-helper");
const rollup_1 = tslib_1.__importDefault(require("rollup"));
const plugin_node_resolve_1 = require("@rollup/plugin-node-resolve");
const plugin_commonjs_1 = tslib_1.__importDefault(require("@rollup/plugin-commonjs"));
const plugin_json_1 = tslib_1.__importDefault(require("@rollup/plugin-json"));
const load_data_url_1 = require("./load-data-url");
class CloudFunctions {
    constructor(pathToSourceCode) {
        this.cloudFunctionsDirectoryPath = path_1.default.join(pathToSourceCode.replace(/^(\.\.(\/|\\|$))+/, ""), constants_1.CLOUD_FUNCTIONS_DIRECTORY);
        this.pathToSourceCode = pathToSourceCode;
    }
    async serve(servingPort) {
        const directoryExists = (0, os_helper_1.checkIfDirectoryExists)(this.cloudFunctionsDirectoryPath);
        if (!directoryExists) {
            throw new cloud_function_errors_1.FunctionsDirectoryNotFoundError(this.pathToSourceCode);
        }
        const cloudFunctionResources = await this.parseCloudFunctionResources();
        const hasCloudFunctionResources = cloudFunctionResources.length;
        if (!hasCloudFunctionResources) {
            console.log("No Serverless functions detected.");
            process.exit(0);
        }
        const cloudFunctionsValidator = new cloud_functions_validator_1.CloudFunctionsValidator(cloudFunctionResources);
        const error = cloudFunctionsValidator.validate();
        if (error) {
            throw error;
        }
        const { exactRouteResources, dynamicRouteResources } = this.transformAndSegregateResourcesByRoutes(cloudFunctionResources);
        const app = (0, express_1.default)();
        app.disable('x-powered-by');
        await this.applyAppRouter(exactRouteResources, app);
        await this.applyAppRouter(dynamicRouteResources, app);
        dotenv_1.default.config({ path: path_1.default.join(this.pathToSourceCode, constants_1.ENV_FILE_NAME) });
        app.listen(servingPort, () => {
            console.log(`Serving on port ${servingPort}`);
        });
    }
    async applyAppRouter(cloudFunctionResources, app) {
        await Promise.all(cloudFunctionResources.map(async (cloudFunctionResource) => {
            app.use(express_1.default.json());
            app.use(express_1.default.urlencoded({ extended: true }));
            app.use((request, response, next) => {
                if (!response.getHeader('cache-control')) {
                    response.setHeader('cache-control', 'no-store');
                }
                next();
            });
            app.all(cloudFunctionResource.apiResourceURI, async (request, response) => {
                try {
                    return await cloudFunctionResource.handler(request, response);
                }
                catch (error) {
                    console.error(error);
                    response.status(500).send();
                }
            });
        }));
    }
    isProxyEdgeFile(filename) {
        const PROXY_EDGE_FILENAME = '[proxy].edge.js';
        return filename === PROXY_EDGE_FILENAME;
    }
    async parseCloudFunctionResources() {
        var _a, e_1, _b, _c;
        const filePaths = await (0, os_helper_1.walkFileSystem)(this.cloudFunctionsDirectoryPath);
        const cloudFunctionResources = [];
        try {
            for (var _d = true, filePaths_1 = tslib_1.__asyncValues(filePaths), filePaths_1_1; filePaths_1_1 = await filePaths_1.next(), _a = filePaths_1_1.done, !_a;) {
                _c = filePaths_1_1.value;
                _d = false;
                try {
                    const filePath = _c;
                    const parsedPath = path_1.default.parse(filePath);
                    if (this.isProxyEdgeFile(parsedPath.base)
                        || parsedPath.ext !== constants_1.CLOUD_FUNCTIONS_SUPPORTED_EXTENSION) {
                        continue;
                    }
                    const handler = await this.buildHandlerForFilepath(filePath);
                    if (!handler) {
                        continue;
                    }
                    const relativeParsedPath = path_1.default.parse(path_1.default.relative(this.cloudFunctionsDirectoryPath, filePath));
                    const apiResourceURI = `/${path_1.default.join(relativeParsedPath.dir, relativeParsedPath.name)}`;
                    cloudFunctionResources.push({
                        cloudFunctionFilePath: filePath,
                        apiResourceURI,
                        handler
                    });
                }
                finally {
                    _d = true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = filePaths_1.return)) await _b.call(filePaths_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return cloudFunctionResources;
    }
    transformAndSegregateResourcesByRoutes(cloudFunctionResources) {
        const matchDyanmicRouteRegex = /\[(.*?)\]/g;
        const exactRouteResources = [];
        const dynamicRouteResources = [];
        if (cloudFunctionResources.length) {
            console.log("Detected Serverless functions...");
        }
        cloudFunctionResources.forEach((cloudFunctionResource) => {
            if (cloudFunctionResource.apiResourceURI.match(matchDyanmicRouteRegex) !==
                null) {
                const apiResourceURI = cloudFunctionResource.apiResourceURI.replace(matchDyanmicRouteRegex, ":$1");
                dynamicRouteResources.push(Object.assign(Object.assign({}, cloudFunctionResource), { apiResourceURI }));
                console.log(`λ ${apiResourceURI} \n`);
            }
            else {
                exactRouteResources.push(cloudFunctionResource);
                console.log(`λ ${cloudFunctionResource.apiResourceURI} \n`);
            }
        });
        return { exactRouteResources, dynamicRouteResources };
    }
    async buildHandlerForFilepath(cloudFunctionFilePath) {
        var _a;
        const bundle = await rollup_1.default.rollup({
            input: cloudFunctionFilePath,
            plugins: [(0, plugin_node_resolve_1.nodeResolve)({ preferBuiltins: true }), (0, plugin_commonjs_1.default)(), (0, plugin_json_1.default)()],
        });
        const { output } = await bundle.generate({
            format: "esm",
            inlineDynamicImports: true,
        });
        const builtCode = output[0].code;
        const builtCodeInDataURLFormat = "data:text/javascript;base64," + Buffer.from(builtCode).toString("base64");
        const module = await (0, load_data_url_1.loadDataURL)(builtCodeInDataURLFormat);
        let handler = null;
        const isDefaultExportESModuleFunction = typeof module.default === 'function';
        const isDefaultExportCommonjsFunction = typeof ((_a = module.default) === null || _a === void 0 ? void 0 : _a.default) === 'function';
        if (isDefaultExportESModuleFunction) {
            handler = module.default;
        }
        else if (isDefaultExportCommonjsFunction) {
            handler = module.default.default;
        }
        return handler;
    }
}
exports.CloudFunctions = CloudFunctions;
